/*
 * Copyright (c) 2020 Raspberry Pi (Trading) Ltd.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef _PICO_STDLIB_H_
#define _PICO_STDLIB_H_

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

#include "piolib.h"

#endif
